clear;
addpath('../../functions')
sfolder='./results';
dfolder=('./data/');
rng(5)

%load data

[data0,names]=xlsread('../../data/benchmark_tax_1.xls');
names=names(1,2:end);

m_t=data0(:,end)*10; %instrument
data0=(data0(:,1:end-1));
mid=isnan(m_t(:,1));  %id for missing obs in instrument
tmp=m_t(:,1);
tmp(mid)=0;
datax=[data0 tmp ];
if sum(mid)>0 %initial estimate for missing instrument data via Kalman filter
    [out,out1]=draw_data_initial1(datax,mid,zeros(1,cols(datax)),eye(cols(datax)),cols(datax),cols(datax)-1,1,0.1);
end
data=data0;
REPS=100000;
BURN=50000;
SKIP=10;
H=40;
CHECK=1;
maxdraws=100;
EX=1;
idx=BURN+1:SKIP:REPS;
fsize=length(idx);
N=cols(data);
npart=10;

L=2;
Y=data;
X=prepare(data,L);
Y=Y(L+1:end,1:N);
X=X(L+1:end,:);
if sum(mid)>0
M=out(L+1:end,end); %intrument 
M0=tmp(L+1:end,:); %instrument withmiss=0
Mx=tmp;
else
    M=m_t(L+1:end,:);
       Mx=m_t;
end
M=detrend(M,'constant');

%setting priors and starting values
T0=40;
y0=Y(1:T0,:);
x0=X(1:T0,:);
b0=x0\y0;
e0=y0-x0*b0;
sigma0=(e0'*e0)/T0;
V0=kron(sigma0,invpd(x0'*x0));
%priors for the variance of the transition equation
Q0=V0*T0*3.5e-04;  %prior for the variance of the transition equation error
P00=10*V0;                    % variance of the intial state vector  variance of state variable p[t-1/t-1]
beta0=vec(b0)';       % intial state vector   %state variable  b[t-1/t-1]
Q=Q0;

%priors and starting values for aij
C0=chol(sigma0);
C0=C0./repmat(diag(C0),1,N);
C0=inv(C0)';
PC0=10;
amatx=zeros(1,N*(N-1)/2);
D0=zeros(N*(N-1)/2,N*(N-1)/2);
j=1;
for i=1:N-1
    temp=C0(i+1,1:i);
amatx(1,j:j+cols(temp)-1)=temp;
D0(j:j+cols(temp)-1,j:j+cols(temp)-1)=eye(cols(temp))*(10^(-4));
j=j+cols(temp);
end
DD=D0;

%priors for instrument equation
% b0m=0.15; %prior mean
% v0m=1;  %prior variance
% meanv=0.01; %prior mean for sig
% variancev=0.00001;%prior variance
% S0=2*meanv*(1+(meanv^2)/(variancev^2));
% NU0=2*(2+(meanv^2)/(variancev^2));


[ VAR01 ] = runproxysimple( data,Mx,10,L );
tmp=packr([VAR01.et M]);
tmpx=tmp(:,1);
tmpy=tmp(:,2);
btmp=tmpx\tmpy;
etmp=tmpy-tmpx*btmp;
stmp=cov(etmp);
b0m=btmp; %prior mean
v0m=0.1;  %prior variance
meanv=btmp/2; %prior mean for sig
variancev=0.02;%0.015;%prior variance
S0=2*meanv*(1+(meanv^2)/(variancev^2));
NU0=2*(2+(meanv^2)/(variancev^2));


beta2m=b0m;
sigmam=meanv;
beta2m0=b0m;
%remove intial Sample
Y=Y(T0+1:end,:);
X=X(T0+1:end,:);
M=M(T0+1:end,:);
mid=mid(T0+L+1:end,:);
if sum(mid)>0
M0=M0(T0+1:end,:);
end
T=rows(X);
amat00=repmat(amatx,T,1);
beta20=repmat(beta0,T,1);
%priors for the initial condition of the stochastic volatility
SS0=10;
g0=10^-4;  %prior scale parameter for inverse gamma
Tg0=1;     %prior degrees of freedom
g=ones(1,N).*g0;
MU0=log(diag(sigma0));
hlast=(diff(Y).^2)+0.0001;   %rough guess for stochastic volatility 
hlast=[hlast(1:2,:);hlast];  %rough intial guess for svol

e0=Y-X*(X\Y) ;
epsilon=e0*C0; %starting values for orthogonolised resids
error=e0; %starting values for VAR residuals
%50 iterations to smooth initial estimates of stochastic volatility

for m=1:50
  hnew=zeros(T+1,N);
  for i=1:N
      htemp=getsvolx(hlast(:,i),g0,MU0(i),SS0,epsilon(:,i));
      hnew(:,i)=htemp;
      
  end
hlast=hnew;
[beta2,error,roots,epsilon,problem]=...
          carterkohnvar(Y,X,Q,amat00,hlast,ones(T,1),beta0,P00,L,0,maxdraws,EX);
 j=1;
for i=2:N
    ytemp=error(:,i);
    xtemp=error(:,1:i-1)*-1;
    ytemp=(ytemp)./sqrt(hlast(2:rows(hlast),i));
    xtemp=(xtemp)./repmat(sqrt(hlast(2:rows(hlast),i)),1,cols(xtemp));
    a0=C0(i,1:i-1);
    pa=PC0*diag(abs(a0));
    Qa=DD(j:j+cols(xtemp)-1,j:j+cols(xtemp)-1);
    [a1,trash]=carterkohn1(a0,pa,ones(T+1,1),Qa,ytemp,xtemp);
    amat00(:,j:j+cols(a1)-1)=a1;
    j=j+cols(a1);
end     
      
      
      
end
amatinv=zeros(T,cols(amat00));
for t=1:T
    tmp=chofac(N,amat00(t,:)');
    tmp1=inv(tmp);
    amatinv(t,:)=unchofac(tmp1)';
end
hlast=hlast(2:end,:);
xfixa=[amatinv log(hlast)]; %initial states: A_t and H_t
beta0a=xfixa(1,:);
BETA01=beta0a;
NS1=cols(BETA01);
tmp=randn(N,1);
tmp=tmp/norm(tmp);

Qold=tmp';
Qstar=Qold;

g=diag(g);
Qbig=blkdiag(D0,g);
NN=length(beta0);
NNx=cols(amat00);
TT0=T0;%NN+1;
TD0=NNx+10;%15;
TG0=N+10;%15;
if sum(mid)>0
    MISS=1;
else
    MISS=0;
end

if MISS
    %Initial values for the Kalman filter B0/0 when drawing missing data
beta0d=[];
for j=0:L-1
    beta0d=[beta0d [Y(L-j,:) M(L-j,:)]];
end
P00d=eye(cols(beta0d));  %P[0/0]
end

save('priors1')

%storage
bmat=zeros(fsize,T,NN);
smat=zeros(fsize,T,N);
amatsave=zeros(fsize,T,(N*(N-1))/2);
gmat=zeros(fsize,N,N);
qa=zeros(fsize,(N*(N-1))/2,(N*(N-1))/2);
qq=zeros(fsize,NN,NN);
bmatm=zeros(fsize,1);
smatm=zeros(fsize,1);
irfsave=zeros(fsize,T,H,N);
irfsave1=irfsave;
a0irfsave=zeros(fsize,T,N);
datasave=zeros(fsize,T,1);

naccept=0;

tic;
gibbs2=1;
gibbs1=1;
gibbs=1;
Q=Q0;
%for gibbs=1:REPS
while gibbs1<=fsize

%step 1: draw beta amat H using the particle gibbs sampler
Fmat=eye(NS1);
Mu=zeros(1,NS1);
Qbig=blkdiag(DD,g);

[betax,BB,W,problem]=...
    pftest100Fr(Fmat,Mu,Qbig,xfixa,Y,X,M,beta2m,sigmam,beta2,repmat(Qstar,T,1),NS1,BETA01,npart,N,L,EX);
index=discretesample(W(:,end),1);
xfixa=betax(:,:,index);
amat=xfixa(:,1:NNx);
hlast=exp(xfixa(:,NNx+1:end));
A0t=zeros(N,N,T);
for t=1:T
    A0=chofac(N,amat(t,:)')*(diag(hlast(t,:).^0.5));
    A0t(:,:,t)=A0;
end

%DRAW TVP parameters VAR


[beta2,error,roots,problemb]=...
carterkohnvar3(Y,X,Q,A0t,beta0,P00,L,CHECK,maxdraws,EX,M,beta2m,sigmam,Qstar);
if problemb
    beta2=beta20;
else
    beta20=beta2;
end


%draw Qstar
 qdraw=randn(N,1);
 qdraw=qdraw/norm(qdraw);
lpnew=0;
lpold=0;

for t=1:T
 A0=A0t(:,:,t);
%  tmpA0irf=(A0*qdraw)';
%  e1=sign(tmpA0irf(1,1))*sign(tmpA0irf(1,2));
 
%  chck=chck+(e1<0);
 lpnewi=loglik_m_given_y(M(t,:), error(t,:), A0, qdraw, beta2m, sigmam.^0.5);
  lpoldi=loglik_m_given_y(M(t,:), error(t,:), A0, Qold', beta2m, sigmam.^0.5);
  if isnan(lpnewi) || isinf(lpnewi) 
      lpnewi=-100;
  end
  
   if isnan(lpoldi) || isinf(lpoldi) 
      lpoldi=-100;
   end
  lpnew=lpnew+lpnewi;
  lpold=lpold+lpoldi;
end
% if chck<T
%     accept=0;
% else
accept=exp(lpnew-lpold);
% end
if accept>rand
      Qold=qdraw';
      naccept=naccept+1;
 end
Qstar=Qold;


epsilon=zeros(T,1);
A0irf=zeros(T,N);
for t=1:T
    A0=A0t(:,:,t);
    epsilon(t,:)=(Qstar / A0)*error(t,:)';
    A0irf(t,:)=(A0*Qstar')';
end
cc=corrcoef([epsilon M]);
check=cc(1,2);%
%normalise sign of epsilon
ee=check(1)<0;
if ee
    Qstar=Qstar*-1;
    epsilon=epsilon*-1;
    Qold=Qold*-1;
    A0irf=A0irf*-1;
end


%draw betam sigmam
 beta2m=getreg(M,epsilon(:,1),b0m,v0m,sigmam);
merrors=M-epsilon(:,1)*beta2m;
NU1=NU0+T;
S1=S0+merrors'*merrors;
ALPHA=NU1/2;
BETA=2/S1;
sigmam=1./gamrnd(ALPHA,BETA);
 %sigmam=ig(merrors,td0m,d0m);


%step 3: Draw Qbig
errord=diff(amat);
scaled=(errord'*errord)+D0;
DD=iwpq(T+TD0,invpd(scaled));

errorh=diff(log(hlast));
scaleh=(errorh'*errorh)+diag(repmat(g0,N,1));
g=iwpq(T+TG0,invpd(scaleh));





% step 2 sample Q from the IW distribution
errorq=diff(beta2);
scaleQ=(errorq'*errorq)+Q0;
Q=iwpq(T+TT0,invpd(scaleQ));

%draw missing data
if MISS
    dataid0=[Y M0];
    
[out,bt]=draw_data5(beta2,A0t,beta2m,sigmam,dataid0,mid,beta0d,P00d,N+1,N,NN,EX,L,Qstar);
M=out(:,1);
end








disp(sprintf('Iteration Number= %s ', num2str([gibbs naccept/gibbs (beta2m^2)/(beta2m^2+sigmam) beta2m sigmam cc(2,1) ee])));

if rem(gibbs,100)==0
    %irf
    irfx=zeros(T,H,N);
    irfx1=irfx;
for t=1:T
    A0=A0t(:,:,t);   
    A0x=zeros(N,N);
    A0x(1,:)=A0irf(t,:);
  shck=zeros(1,N);
    shck(1)=1;
   imp=irfsim(reshape(beta2(t,:),N*L+EX,N),N,L,A0x,shck,H+L);
irfx1(t,:,:)=imp;
irfx(t,:,:)=imp./imp(1,1);

end

    save('progress','hlast','amat','beta2m','sigmam','A0irf','roots','epsilon','irfx1')
end

if gibbs>BURN
   
   if sum(gibbs==idx)>0

    bmat(gibbs1,:,:)=beta2;
smat(gibbs1,:,:)=hlast;
amatsave(gibbs1,:,:)=amat;
qa(gibbs1,:,:)=DD;
gmat(gibbs1,:,:)=g;
qq(gibbs1,:,:)=Q;
bmatm(gibbs1,:)=beta2m;
smatm(gibbs1,:)=sigmam';
if MISS
    datasave(gibbs1,:)=M;
end
%irf
%irf
    irfx=zeros(T,H,N);
    irfx1=irfx;
    
for t=1:T
    A0=A0t(:,:,t);
    A0x=zeros(N,N);
    A0x(1,:)=A0irf(t,:);
    shck=zeros(1,N);
    shck(1)=1;
   imp=irfsim(reshape(beta2(t,:),N*L+EX,N),N,L,A0x,shck,H+L);

mse=0;
for j=1:N
    shck=zeros(1,N);
    shck(j)=1;
   impx=irfsim(reshape(beta2(t,:),N*L+EX,N),N,L,A0',shck,H+L);
   mse=mse+cumsum(impx.^2);
end

 irfx1(t,:,:)=cumsum(imp.^2)./mse;
irfx(t,:,:)=imp./imp(1,1);   
end





irfsave(gibbs1,:,:,:)=irfx;
irfsave1(gibbs1,:,:,:)=irfx1;

a0irfsave(gibbs1,:,:)=A0irf;

gibbs1=gibbs1+1;
   end
    end

  

gibbs=gibbs+1;
end
toc;
save(strcat('results',num2str(1)),'bmat','smat','amatsave','qa','gmat','qq','smat','irfsave','irfsave1','bmatm','smatm','a0irfsave');


