function prior = priorsim( pshape,p1,p2,lb,ub, ndraws)

% priorsim.m
% This function simulates ndraws from the prior distribution used for
% generating the prior density plot
% The code comes with no performance guarantees.
% However, if you find this code useful then please cite the paper.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% July 2005: orginal author Philip Liu
%
%==========================================================================
% Input:
%       pshape = parameter distributions of size n
%       p1 = mean
%       p2 = std deviations
%       ndraws = number of draws generated
%
% Return: 
%       prior = n x ndraws matrix containing prior draws
%       
%==========================================================================
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

npara = size(pshape,1);
prior=[];
% pshape = parainfo(:,1);
% p1 = parainfo(:,2);
% p2 = parainfo(:,3);
% lb = parainfo(:,4);
% ub = parainfo(:,5);
ii=1;
while ii <= npara
    a=0;
    b=0;
    check=0;
    if pshape(ii) == 0;     % no prior
        draws = ones(ndraws,1)*p1(ii);
        prior=[prior draws];
        
    elseif pshape(ii) == 1;     %  BETA Prior
        a = (1-p1(ii))*p1(ii)^2/p2(ii)^2 - p1(ii);
        b = a*(1/p1(ii) - 1);
        while check==0;
            draws=betarnd(a,b,ndraws,1);
            if draws<lb(ii)|draws>ub(ii)
                check=0;
            else
                check=1;
            end
        end

        prior=[prior draws];


    elseif pshape(ii) == 2; % GAMMA PRIOR
        a = p1(ii)^2/p2(ii)^2;
        b = p2(ii)^2/p1(ii);
        while check==0;
            draws=gamrnd(a,b,ndraws,1);
            if draws<lb(ii)|draws>ub(ii)
                check=0;
            else
                check=1;
            end
        end

        prior=[prior draws];

    elseif pshape(ii) == 3; % GAUSSIAN PRIOR
        while check==0;
            draws=normrnd(p1(ii),p2(ii),ndraws,1);
            if draws<lb(ii)|draws>ub(ii)
                check=0;
            else
                check=1;
            end
        end

        prior=[prior draws];

    elseif pshape(ii) == 4; % INVGAMMA PRIOR
        a = p1(ii)^2/p2(ii)^2 + 2;
        b = p1(ii)*(a-1);
        while check==0;
            draws=1./gamrnd(a,1/b,ndraws,1);
            if draws<lb(ii)|draws>ub(ii)
                check=0;
            else
                check=1;
            end
        end

        prior=[prior draws];

    elseif pshape(ii) == 5; % dirichlet
        draws=[]
        for m=1:ndraws
        %while check==0;
            draws1= rand_d([p1(ii);p2(ii)]);
%             draws1(1)
%             if draws1(1)<lb(ii)|draws1(1)>ub(ii)
%                 check=0;
%             else
%                 check=1;
%             end
%         end
        
        draws=[draws;draws1(1)'];
        end

        prior=[prior draws];
    end;
    ii = ii+1;
end;
