function forecast=runUC(DATAin,REPS,BURN,HORIZONin,T0,filex)
%inputs
data=packr(DATAin);
reps=REPS;%total reps
burn=BURN;%burn in
HORIZON=HORIZONin;%forecast horizon
%
N=1;
%output=forecast density
forecast=zeros(reps-burn,HORIZON,N);

Y=data;
pmat=data-hpfilter(data,1600);



%step 1 set starting values and priors using a pre-sample of 10 years
y0=Y(1:T0,:);
x0=pmat(1:T0,:);
b0=x0\y0;
e0=y0-x0*b0;
sigma0=(e0'*e0)/T0;

Y=Y(T0+1:end,:);
pmat=pmat(T0+1:end,:);
T=rows(Y);


%priors and starting values for the stochastic vol R
hlast=(diff(Y).^2)+0.0001;
hlast=[hlast(1:2,:);hlast];  %rough intial guess for svol
g=ones(N,1);  %rough guess for the variance of the transition equation
g0=0.1^2;  %scale parameter for inverse gamma
Tg0=1;
mubar=log(diag(sigma0));
sigmabar=10;


%priors and starting values for stochastic volatility Q
mubarq=log(diag(std(pmat)^2));
sigmabarq=10;
hlastq=(diff(pmat).^2)+0.0001;
hlastq=[hlastq(1:2,:);hlastq];  %rough intial guess for svol
gq=ones(N,1);  %rough guess for the variance of the transition equation
g0q=0.01^2;  %scale parameter for inverse gamma
Tg0q=1;


%initialise parameters
bf00=pmat(1,:);
pf00=eye(cols(bf00));
resR=Y-pmat;
resQ=diff(pmat);
resQ=[resQ(1,:);resQ];

%Gibbs sampling algorithm 
index=1:100:reps;

tic;
mm=1;
m=1;
while mm<(reps-burn)+1
    
   %step 1 Sample R
   hnew=getsvolx(hlast,g,mubar,sigmabar,resR);
   hlast=hnew;
   gerrors=diff(log(hnew));
    g =ig(gerrors,Tg0,g0);  %draw from the inverse Gamma distribution
    
    %step 2 Sample Q
   hnewq=getsvolx(hlastq,gq,mubarq,sigmabarq,resQ);
   hlastq=hnewq;
   gerrorsq=diff(log(hnewq));
    gq =ig(gerrorsq,Tg0q,g0q);  %draw from the inverse Gamma distribution
   
    %step 3 trend component
    
    
% if sum(m==index)>0
% disp(sprintf('Iteration Number= %s ', num2str(m)));
% end
%step 1 sample VAR coefficients
[beta2]=carterkohnUC(Y,bf00,pf00,hlast,hlastq);

%update errors
resR=Y-beta2;
resQ=diff(beta2);
resQ=[resQ(1,:);resQ];

if sum(m==index)>0
disp(sprintf('Iteration Number= %s file=%s ', num2str(m),num2str(filex)));
end



if m>=burn
    
    %Produce Forecast
    L=1;
    yhat=zeros(L+HORIZON,N);
    yhat(1:L,:)=Y(T:T,:);
    betahat=zeros(L+HORIZON,1);
    betahat(L,:)=beta2(T,:);
    
    hhat=zeros(L+HORIZON,1);
    hhat(L,:)=log(hlast(T+1,:));
    hhatq=zeros(L+HORIZON,1);
    hhatq(L,:)=log(hlastq(T+1,:));
    
    gc=(sqrt(g));
    gcq=sqrt(gq);
   
    for i=L+1:HORIZON+L
        hhat(i,:)=hhat(i-1)+randn(1,1)*gc;
        hhatq(i,:)=hhatq(i-1)+randn(1,1)*gcq;
       
        betahat(i,:)=betahat(i-1,:)+randn(1,1)*sqrt(exp(hhatq(i,:)));
        %forecast
       
        yhat(i,:)=betahat(i,:)+randn(1,1)*sqrt(exp(hhat(i,:)));
    end
            
     forecast(mm,:,:)=yhat(L+1:end,:);   
        
        
    
    
    mm=mm+1;
    end
    

m=m+1;
end
toc;
